@@ -8,15 +8,14 @@ from .layers import (DecoderLayer, EncoderLayer, MultiHeadAttention,
 from .losses import MaskedSparseCategoricalCrossentropy
 from .metrics import MaskedSparseCategoricalAccuracy
 from .modeling_albert import (Albert, AlbertEmbedding, AlbertEncoder,
-                              AlbertEncoderGroup, AlbertEncoderLayer,
-                              AlbertMLMHead, AlbertSOPHead)
+                              AlbertEncoderGroup, AlbertEncoderLayer)
 from .modeling_bert import (Bert, BertEmbedding, BertEncoder, BertEncoderLayer,
-                            BertIntermediate, BertMLMHead, BertNSPHead)
+                            BertIntermediate)
 from .modeling_utils import complete_inputs
 from .tokenizers.bert_tokenizer import BertTokenizer
 from .tokenizers.tokenizer import BasicTokenizer, WordpieceTokenizer
 
 __name__ = 'transformers_keras'
-__version__ = '0.2.6'
+__version__ = '0.3.0'
 
 logging.basicConfig(format="%(asctime)s %(levelname)7s %(filename)15s %(lineno)4d] %(message)s", level=logging.INFO)
